@@ -32,7 +32,7 @@ class DataSet(object):
 
         # Introspect the database and generate models.
         self._introspector = Introspector.from_database(self._database)
-        self._models = self._introspector.generate_models()
+        self._models = self._introspector.generate_models(skip_invalid=True)
         self._migrator = SchemaMigrator.from_database(self._database)
 
         class BaseModel(Model):
@@ -71,6 +71,17 @@ class DataSet(object):
     def close(self):
         self._database.close()
 
+    def update_cache(self, table=None):
+        if table:
+            model_class = self._models[table]
+            dependencies = model_class._meta.related_models(backrefs=True)
+        else:
+            dependencies = None  # Update all tables.
+        updated = self._introspector.generate_models(
+            skip_invalid=True,
+            table_names=[related._meta.db_table for related in dependencies])
+        self._models.update(updated)
+
     def __enter__(self):
         self.connect()
         return self
@@ -136,7 +147,9 @@ class Table(object):
             model_class.create_table()
             self.dataset._models[name] = model_class
 
-        self.model_class = model_class
+    @property
+    def model_class(self):
+        return self.dataset._models[self.name]
 
     def __repr__(self):
         return '<Table: %s>' % self.name
@@ -188,6 +201,8 @@ class Table(object):
 
             migrate(*operations)
 
+            self.dataset.update_cache(self.name)
+
     def insert(self, **data):
         self._migrate_new_columns(data)
         return self.model_class.insert(**data).execute()
