@@ -1,3 +1,4 @@
+from collections import namedtuple
 import re
 
 from peewee import *
@@ -32,14 +33,15 @@ class Column(object):
     primary_key_types = (IntegerField, PrimaryKeyField)
 
     def __init__(self, name, field_class, raw_column_type, nullable,
-                 primary_key=False, max_length=None, db_column=None):
+                 primary_key=False, db_column=None, index=False, unique=False):
         self.name = name
         self.field_class = field_class
         self.raw_column_type = raw_column_type
         self.nullable = nullable
         self.primary_key = primary_key
-        self.max_length = max_length
         self.db_column = db_column
+        self.index = index
+        self.unique = unique
 
     def __repr__(self):
         attrs = [
@@ -47,7 +49,6 @@ class Column(object):
             'raw_column_type',
             'nullable',
             'primary_key',
-            'max_length',
             'db_column']
         keyword_args = ', '.join(
             '%s=%s' % (attr, getattr(self, attr))
@@ -60,19 +61,24 @@ class Column(object):
         # Set up default attributes.
         if self.nullable:
             params['null'] = True
-        if self.field_class is CharField and self.max_length:
-            params['max_length'] = self.max_length
         if self.field_class is ForeignKeyField or self.name != self.db_column:
             params['db_column'] = "'%s'" % self.db_column
         if self.primary_key and not self.field_class is PrimaryKeyField:
             params['primary_key'] = True
 
         # Handle ForeignKeyField-specific attributes.
-        if self.field_class is ForeignKeyField:
+        if self.is_foreign_key():
             params['rel_model'] = self.rel_model
             if self.to_field:
                 params['to_field'] = "'%s'" % self.to_field
 
+        # Handle indexes on column.
+        if not self.is_primary_key():
+            if self.unique:
+                params['unique'] = 'True'
+            elif self.index and not self.is_foreign_key():
+                params['index'] = 'True'
+
         return params
 
     def is_primary_key(self):
@@ -110,21 +116,6 @@ class Column(object):
         return field
 
 
-class ForeignKeyMapping(object):
-    def __init__(self, table, column, dest_table, dest_column):
-        self.table = table
-        self.column = column
-        self.dest_table = dest_table
-        self.dest_column = dest_column
-
-    def __repr__(self):
-        return 'ForeignKeyMapping(%s.%s -> %s.%s)' % (
-            self.table,
-            self.column,
-            self.dest_table,
-            self.dest_column)
-
-
 class Metadata(object):
     column_map = {}
 
@@ -134,25 +125,47 @@ class Metadata(object):
     def execute(self, sql, *params):
         return self.database.execute_sql(sql, params)
 
-    def set_search_path(self, *path):
-        self.database.set_search_path(*path)
+    def get_columns(self, table, schema=None):
+        metadata = dict(
+            (metadata.name, metadata)
+            for metadata in self.database.get_columns(table, schema))
 
-    def get_tables(self):
-        """Returns a list of table names."""
-        return self.database.get_tables()
+        # Look up the actual column type for each column.
+        column_types = self.get_column_types(table, schema)
 
-    def get_columns(self, table):
-        pass
+        # Look up the primary keys.
+        pk_names = self.get_primary_keys(table, schema)
+        if len(pk_names) == 1:
+            pk = pk_names[0]
+            if column_types[pk] is IntegerField:
+                column_types[pk] = PrimaryKeyField
 
-    def get_primary_keys(self, table, schema=None):
-        pass
+        columns = {}
+        for name, column_data in metadata.items():
+            columns[name] = Column(
+                name,
+                field_class=column_types[name],
+                raw_column_type=column_data.data_type,
+                nullable=column_data.null,
+                primary_key=column_data.primary_key,
+                db_column=name)
+
+        return columns
+
+    def get_column_types(self, table, schema=None):
+        raise NotImplementedError
 
     def get_foreign_keys(self, table, schema=None):
-        pass
+        return self.database.get_foreign_keys(table, schema)
+
+    def get_primary_keys(self, table, schema=None):
+        return self.database.get_primary_keys(table, schema)
+
+    def get_indexes(self, table, schema=None):
+        return self.database.get_indexes(table, schema)
 
 
 class PostgresqlMetadata(Metadata):
-    # select oid, typname from pg_type;
     column_map = {
         16: BooleanField,
         17: BlobField,
@@ -189,87 +202,36 @@ class PostgresqlMetadata(Metadata):
                 elif 'tsvector' in typname:
                     self.column_map[oid] = postgres_ext.TSVectorField
 
-    def get_columns(self, table):
-        # Get basic metadata about columns.
-        cursor = self.execute("""
-            SELECT
-                column_name, is_nullable, data_type, character_maximum_length
-            FROM information_schema.columns
-            WHERE table_name=%s""", table)
-        name_to_info = {}
-        for row in cursor.fetchall():
-            name_to_info[row[0]] = {
-                'db_column': row[0],
-                'nullable': row[1] == 'YES',
-                'raw_column_type': row[2],
-                'max_length': row[3],
-                'primary_key': False,
-            }
+    def get_column_types(self, table, schema):
+        column_types = {}
 
         # Look up the actual column type for each column.
-        cursor = self.execute('SELECT * FROM "%s" LIMIT 1' % table)
+        identifier = '"%s"."%s"' % (schema, table)
+        cursor = self.execute('SELECT * FROM %s LIMIT 1' % identifier)
 
         # Store column metadata in dictionary keyed by column name.
         for column_description in cursor.description:
-            field_class = self.column_map.get(
+            column_types[column_description.name] = self.column_map.get(
                 column_description.type_code,
                 UnknownField)
-            column = column_description.name
-            name_to_info[column]['field_class'] = field_class
 
-        # Look up the primary keys.
-        pk_names = self.get_primary_keys(table)
-        if len(pk_names) == 1:
-            for pk_name in pk_names:
-                name_to_info[pk_name]['primary_key'] = True
-                if name_to_info[pk_name]['field_class'] is IntegerField:
-                    name_to_info[pk_name]['field_class'] = PrimaryKeyField
+        return column_types
 
-        columns = {}
-        for name, column_info in name_to_info.items():
-            columns[name] = Column(
-                name,
-                field_class=column_info['field_class'],
-                raw_column_type=column_info['raw_column_type'],
-                nullable=column_info['nullable'],
-                primary_key=column_info['primary_key'],
-                max_length=column_info['max_length'],
-                db_column=name)
-
-        return columns
-
-    def get_primary_keys(self, table, schema=None):
-        cursor = self.execute("""
-            SELECT pg_attribute.attname
-            FROM pg_index, pg_class, pg_attribute
-            WHERE
-              pg_class.oid = '%s'::regclass AND
-              indrelid = pg_class.oid AND
-              pg_attribute.attrelid = pg_class.oid AND
-              pg_attribute.attnum = any(pg_index.indkey)
-              AND indisprimary;""" % table)
-        return set(row[0] for row in cursor.fetchall())
+    def get_columns(self, table, schema=None):
+        schema = schema or 'public'
+        return super(PostgresqlMetadata, self).get_columns(table, schema)
 
     def get_foreign_keys(self, table, schema=None):
         schema = schema or 'public'
-        sql = """
-            SELECT
-                kcu.column_name, ccu.table_name, ccu.column_name
-            FROM information_schema.table_constraints AS tc
-            JOIN information_schema.key_column_usage AS kcu
-                ON (tc.constraint_name = kcu.constraint_name AND
-                    tc.constraint_schema = kcu.constraint_schema)
-            JOIN information_schema.constraint_column_usage AS ccu
-                ON (ccu.constraint_name = tc.constraint_name AND
-                    ccu.constraint_schema = tc.constraint_schema)
-            WHERE
-                tc.constraint_type = 'FOREIGN KEY' AND
-                tc.table_name = %s AND
-                tc.table_schema = %s"""
-        cursor = self.execute(sql, table, schema)
-        return [
-            ForeignKeyMapping(table, column, dest_table, dest_column)
-            for column, dest_table, dest_column in cursor]
+        return super(PostgresqlMetadata, self).get_foreign_keys(table, schema)
+
+    def get_primary_keys(self, table, schema=None):
+        schema = schema or 'public'
+        return super(PostgresqlMetadata, self).get_primary_keys(table, schema)
+
+    def get_indexes(self, table, schema=None):
+        schema = schema or 'public'
+        return super(PostgresqlMetadata, self).get_indexes(table, schema)
 
 
 class MySQLMetadata(Metadata):
@@ -304,24 +266,8 @@ class MySQLMetadata(Metadata):
             kwargs['passwd'] = kwargs.pop('password')
         super(MySQLMetadata, self).__init__(database, **kwargs)
 
-    def get_columns(self, table):
-        primary_keys = self.get_primary_keys(table)
-
-        # Get basic metadata about columns.
-        cursor = self.execute("""
-            SELECT
-                column_name, is_nullable, data_type, character_maximum_length
-            FROM information_schema.columns
-            WHERE table_name=%s AND table_schema=DATABASE()""", table)
-        name_to_info = {}
-        for row in cursor.fetchall():
-            name_to_info[row[0]] = {
-                'db_column': row[0],
-                'nullable': row[1] == 'YES',
-                'raw_column_type': row[2],
-                'max_length': row[3],
-                'primary_key': False,
-            }
+    def get_column_types(self, table, schema=None):
+        column_types = {}
 
         # Look up the actual column type for each column.
         cursor = self.execute('SELECT * FROM `%s` LIMIT 1' % table)
@@ -329,46 +275,9 @@ class MySQLMetadata(Metadata):
         # Store column metadata in dictionary keyed by column name.
         for column_description in cursor.description:
             name, type_code = column_description[:2]
-            field_class = self.column_map.get(type_code, UnknownField)
+            column_types[name] = self.column_map.get(type_code, UnknownField)
 
-            if name in primary_keys and len(primary_keys) == 1:
-                name_to_info[name]['primary_key'] = True
-                if field_class is IntegerField:
-                    field_class = PrimaryKeyField
-
-            name_to_info[name]['field_class'] = field_class
-
-        columns = {}
-        for name, column_info in name_to_info.items():
-            columns[name] = Column(
-                name,
-                field_class=column_info['field_class'],
-                raw_column_type=column_info['raw_column_type'],
-                nullable=column_info['nullable'],
-                primary_key=column_info['primary_key'],
-                max_length=column_info['max_length'],
-                db_column=name)
-
-        return columns
-
-    def get_primary_keys(self, table, schema=None):
-        primary_keys = set()
-        cursor = self.execute('SHOW INDEX FROM `%s`' % table)
-        return set(row[4] for row in cursor.fetchall() if row[2] == 'PRIMARY')
-
-    def get_foreign_keys(self, table, schema=None):
-        framing = """
-            SELECT column_name, referenced_table_name, referenced_column_name
-            FROM information_schema.key_column_usage
-            WHERE table_name = %s
-                AND table_schema = DATABASE()
-                AND referenced_table_name IS NOT NULL
-                AND referenced_column_name IS NOT NULL
-        """
-        cursor = self.execute(framing, table)
-        return [
-            ForeignKeyMapping(table, column, dest_table, dest_column)
-            for column, dest_table, dest_column in cursor]
+        return column_types
 
 
 class SqliteMetadata(Metadata):
@@ -411,81 +320,24 @@ class SqliteMetadata(Metadata):
         else:
             column_type = re.sub('\(.+\)', '', raw_column_type)
             field_class = self.column_map.get(column_type, UnknownField)
-        return field_class, raw_column_type
+        return field_class
 
-    def get_columns(self, table):
-        pks = self.get_primary_keys(table)
-        columns = {}
+    def get_column_types(self, table, schema=None):
+        column_types = {}
+        columns = self.database.get_columns(table)
 
-        # Column ID, Name, Column Type, Not Null?, Default, Is Primary Key?
-        cursor = self.execute('PRAGMA table_info("%s")' % table)
-        results = cursor.fetchall()
+        for column in columns:
+            column_types[column.name] = self._map_col(column.data_type)
 
-        for (_, name, column_type, not_null, _, is_pk) in results:
-            field_class, raw_column_type = self._map_col(column_type)
+        return column_types
 
-            if is_pk and len(pks) > 1:
-                is_pk = False
-            elif is_pk and field_class == IntegerField:
-                field_class = PrimaryKeyField
 
-            max_length = None
-            if field_class is CharField:
-                match = re.match('\w+\((\d+)\)', column_type)
-                if match:
-                    max_length, = match.groups()
-
-            columns[name] = Column(
-                name,
-                field_class=field_class,
-                raw_column_type=raw_column_type,
-                nullable=not not_null,
-                primary_key=is_pk,
-                max_length=max_length,
-                db_column=name)
-
-        return columns
-
-    def get_primary_keys(self, table, schema=None):
-        cursor = self.execute('PRAGMA table_info("%s")' % table)
-        return set(row[1] for row in cursor.fetchall() if row[-1])
-
-    def get_foreign_keys(self, table, schema=None):
-        query = """
-            SELECT sql
-            FROM sqlite_master
-            WHERE (tbl_name = ? AND type = ?)"""
-        cursor = self.execute(query, table, 'table')
-        table_definition = cursor.fetchone()[0].strip()
-
-        try:
-            columns = re.search(
-                '\((.+)\)',
-                table_definition,
-                re.MULTILINE | re.DOTALL).groups()[0]
-        except AttributeError:
-            raise ValueError(
-                'Unable to read table definition for "%s"' % table)
-
-        # Replace any new-lines or other junk with whitespace.
-        columns = re.sub('[\s\n\r]+', ' ', columns).strip()
-
-        fks = []
-        for column_def in columns.split(','):
-            column_def = column_def.strip()
-            match = re.search(self.re_foreign_key, column_def, re.I)
-            if not match:
-                continue
-
-            column, dest_table, dest_column = [
-                s.strip('"') for s in match.groups()]
-            fks.append(ForeignKeyMapping(
-                table=table,
-                column=column,
-                dest_table=dest_table,
-                dest_column=dest_column))
-
-        return fks
+DatabaseMetadata = namedtuple('DatabaseMetadata', (
+    'columns',
+    'primary_keys',
+    'foreign_keys',
+    'model_names',
+    'indexes'))
 
 
 class Introspector(object):
@@ -527,9 +379,11 @@ class Introspector(object):
             column += '_'
         return column
 
-    def introspect(self):
+    def introspect(self, table_names=None):
         # Retrieve all the tables in the database.
-        tables = self.metadata.get_tables()
+        tables = self.metadata.database.get_tables()
+        if table_names is not None:
+            tables = [table for table in tables if table in table_names]
 
         # Store a mapping of table name -> dictionary of columns.
         columns = {}
@@ -543,9 +397,13 @@ class Introspector(object):
         # Store a mapping of table name -> model name.
         model_names = {}
 
+        # Store a mapping of table name -> indexes.
+        indexes = {}
+
         # Gather the columns for each table.
         for table in tables:
-            columns[table] = self.metadata.get_columns(table)
+            table_indexes = self.metadata.get_indexes(table, self.schema)
+            table_columns = self.metadata.get_columns(table, self.schema)
             try:
                 foreign_keys[table] = self.metadata.get_foreign_keys(
                     table, self.schema)
@@ -555,11 +413,20 @@ class Introspector(object):
 
             model_names[table] = self.make_model_name(table)
 
-            for column_name, column in columns[table].items():
+            for column_name, column in table_columns.items():
                 column.name = self.make_column_name(column_name)
 
+            for index in table_indexes:
+                if len(index.columns) == 1:
+                    column = index.columns[0]
+                    if column in table_columns:
+                        table_columns[column].unique = index.unique
+                        table_columns[column].index = True
+
             primary_keys[table] = self.metadata.get_primary_keys(
                 table, self.schema)
+            columns[table] = table_columns
+            indexes[table] = table_indexes
 
         # On the second pass convert all foreign keys.
         for table in tables:
@@ -576,10 +443,15 @@ class Introspector(object):
                     model_names,
                     dest)
 
-        return columns, primary_keys, foreign_keys, model_names
+        return DatabaseMetadata(
+            columns,
+            primary_keys,
+            foreign_keys,
+            model_names,
+            indexes)
 
-    def generate_models(self):
-        columns, primary_keys, foreign_keys, model_names = self.introspect()
+    def generate_models(self, skip_invalid=False, table_names=None):
+        database = self.introspect(table_names=table_names)
         models = {}
 
         class BaseModel(Model):
@@ -587,30 +459,42 @@ class Introspector(object):
                 database = self.metadata.database
 
         def _create_model(table, models):
-            for foreign_key in foreign_keys[table]:
+            for foreign_key in database.foreign_keys[table]:
                 dest = foreign_key.dest_table
 
                 if dest not in models and dest != table:
                     _create_model(dest, models)
 
             primary_keys = []
-            for db_column, column in columns[table].items():
+            columns = database.columns[table]
+            for db_column, column in columns.items():
                 if column.primary_key:
                     primary_keys.append(column.name)
 
+            multi_column_indexes = []
+            column_indexes = {}
+            indexes = database.indexes[table]
+            for index in indexes:
+                if len(index.columns) > 1:
+                    field_names = [columns[column].name
+                                   for column in index.columns]
+                    multi_column_indexes.append((field_names, index.unique))
+                elif len(index.columns) == 1:
+                    column_indexes[index.columns[0]] = index.unique
+
             class Meta:
-                pass
+                indexes = multi_column_indexes
 
             # Fix models with multi-column primary keys.
             if len(primary_keys) == 0:
-                primary_keys = columns[table].keys()
+                primary_keys = columns.keys()
             if len(primary_keys) > 1:
                 Meta.primary_key = CompositeKey([
-                    field.name for col, field in columns[table].items()
+                    field.name for col, field in columns.items()
                     if col in primary_keys])
 
             attrs = {'Meta': Meta}
-            for db_column, column in columns[table].items():
+            for db_column, column in columns.items():
                 FieldClass = column.field_class
                 if FieldClass is UnknownField:
                     FieldClass = BareField
@@ -618,11 +502,9 @@ class Introspector(object):
                 params = {
                     'db_column': db_column,
                     'null': column.nullable}
-                if FieldClass is CharField and column.max_length:
-                    params['max_length'] = int(column.max_length)
                 if column.primary_key and not FieldClass is PrimaryKeyField:
                     params['primary_key'] = True
-                if FieldClass is ForeignKeyField:
+                if column.is_foreign_key():
                     if column.is_self_referential_fk():
                         params['rel_model'] = 'self'
                     else:
@@ -633,13 +515,22 @@ class Introspector(object):
 
                     # Generate a unique related name.
                     params['related_name'] = '%s_%s_rel' % (table, db_column)
+                if db_column in column_indexes and not column.is_primary_key():
+                    if column_indexes[db_column]:
+                        params['unique'] = True
+                    elif not column.is_foreign_key():
+                        params['index'] = True
 
                 attrs[column.name] = FieldClass(**params)
 
-            models[table] = type(str(table), (BaseModel,), attrs)
+            try:
+                models[table] = type(str(table), (BaseModel,), attrs)
+            except ValueError:
+                if not skip_invalid:
+                    raise
 
         # Actually generate Model classes.
-        for table, model in sorted(model_names.items()):
+        for table, model in sorted(database.model_names.items()):
             if table not in models:
                 _create_model(table, models)
 
